/* Abstract Object Interface */
#ifndef Ig_ABSTRACT_H
#define Ig_ABSTRACT_H

#if defined(__cplusplus)
extern "C"
{
#endif

/* 
 * Object protocol interface 
 * =========================
 */

/* Generic allocation/deallocation operations on objects. */
Ig_API(IgObject *) IgObject_New(IgTypeObject *type);
Ig_API(int) IgObject_Del(IgObject *o);

/* 
 * Print an object on stdout. Returns -1 on error. This is the equivalent
 * of the Iguana statement: print o.
 */
Ig_API(int) IgObject_Print(IgObject *o);

/*
 * Print the object o to a file fp. The object's print method is called 
 * (if provided), otherwise this function returns -1. 
 */
Ig_API(int) IgObject_PrintFile(IgObject *o, FILE *fp);

/*
 * Compute a string representation of object o. Returns the string
 * representation on success, NULL on failure. This is the
 * equivalent of the Iguana expression "repr(o)". 
 */
Ig_API(IgObject *) IgObject_Repr(IgObject *o);

/*
 * Compare the values of the object o1 and o2 using a routine provided by o1. 
 * Numeric values are coerced.
 * Returns the result of the comparison on success. On error, the value 
 * returned must be ignored (-1) and an exception is raised.
 *
 * The operator is provided by the third column. This is equivalent to 
 * the following Iguana expressions: 
 * 
 *      Expr      Value         
 *      ======    ========
 *	o1 < o2	  IgCmp_LT
 *      o1 > o2	  IgCmp_GT
 *      o1 <= o2  IgCmp_LE
 * 	o1 >= o2  IgCmp_GE
 *	o1 != o2  IgCmp_NE
 *	o1 == o2  IgCmp_EQ
 */
Ig_API(int) IgObject_Cmp(IgObject *o1, IgObject *o2, IgCmp op);

/* Helper: transform a comparison code to a human-readable string */
Ig_API(const char *) IgCmp_Op2Str(IgCmp op);

/*
 * Returns 1 if the object, o, is considered to be true, 0 if o is
 * considered to be false and -1 on failure. This is equivalent to
 * the Iguana expression: not not o
 */
Ig_API(int) IgObject_IsTrue(IgObject *o);

/* 
 * Returns 0 if the object, o, is considered to be true, 1 if o is
 * considered to be false and -1 on failure. This is equivalent to 
 * the Iguana expression: not o
 */
Ig_API(int) IgObject_IsFalse(IgObject *o);

/*
 * Returns 1 if the object, o, is considered to be callable, 0 otherwise.
 * This function always succeeds.
 */
Ig_API(int) IgObject_IsCallable(IgObject *o);


/* 
 * Number protocol interface 
 * =========================
 */

/*
 * Returns 1 if the object o provides numeric protocols, and
 * 0 otherwise.
 *
 * This macro always succeeds.
 */
#define IgNumber_TYPE(o)      ((o) && (o)->ob_type->tp_as_number)

/* 
 * Coerce two numeric types to the "larger" one.
 * Decrement the reference count on the "coerced" argument.
 * 
 * Return values:
 *  -1 	if an error occurred (and an exception is raised)
 *   0  if the coercion succeeded 
 */
Ig_API(int) IgNumber_Coerce(IgObject **pv, IgObject **pu);

/*
 * Returns the result of adding v and u, or NULL on failure.
 * This is the equivalent of the Iguana expression: v+u
 */
Ig_API(IgObject *) IgNumber_Add(IgObject *v, IgObject *u);

/*
 * Returns the result of subtracting u from v, or NULL on failure.
 * This is the equivalent of the Iguana expression: v-u
 */
Ig_API(IgObject *) IgNumber_Subtract(IgObject *v, IgObject *u);

/*
 * Returns the result of multiplying v and u, or NULL on
 * failure. This is the equivalent of the Iguana expression: v*u
 */
Ig_API(IgObject *) IgNumber_Multiply(IgObject *v, IgObject *u);

/*
 * Returns the result of dividing v by u, or NULL on failure.
 * This is the equivalent of the Iguana expression: v/u
 * 
 * The table below shows any combination of input and their return type:
 * 
 *   Op1     Op2      Result
 *   ===     ===      ======
 *   int     int      int
 *   int     float    float 
 *   float   int      float
 *   float   float    float
 */
Ig_API(IgObject *) IgNumber_TrueDivide(IgObject *v, IgObject *u);

/*
 * Returns the result of dividing v by u, giving an integral result or NULL 
 * on failure. This is the equivalent of the Iguana expression: v div u
 */
Ig_API(IgObject *) IgNumber_FloorDivide(IgObject *v, IgObject *u);

/*
 * Returns the remainder of dividing v by u, or NULL on
 * failure. This is the equivalent of the Iguana expression: v mod u
 */
Ig_API(IgObject *) IgNumber_Remainder(IgObject *v, IgObject *u);

/*
 * Return v raised to the power of u. Returns NULL on failure.
 * This is the equivalent of the Iguana expression: v**u
 */
Ig_API(IgObject *) IgNumber_Power(IgObject *v, IgObject *u);

/*
 * Returns v on success, or NULL on failure.
 * This is the equivalent of the Iguana expression: +v
 */
Ig_API(IgObject *) IgNumber_Positive(IgObject *v);

/*
 * Returns the negation of v on success, or NULL on failure.
 * This is the equivalent of the Iguana expression: -v
 */
Ig_API(IgObject *) IgNumber_Negative(IgObject *v);

/*
 * Returns the absolute value of v on success, or NULL on failure.
 * This is the equivalent of the Iguana expression: |v|
 */
Ig_API(IgObject *) IgNumber_Absolute(IgObject *v);

/*
 * Returns 1 if v is zero, 0 otherwise and -1 on failure. This is equivalent 
 * to the Iguana expression: v == 0
 */
Ig_API(int) IgNumber_IsZero(IgObject *v);

/*
 * Returns 1 if v is positive, 0 otherwise and -1 on failure. This is equivalent
 * to the Iguana expression: v > 0
 */
Ig_API(int) IgNumber_IsPositive(IgObject *v);

/*
 * Returns 1 if v is negative, 0 otherwise and -1 on failure. This is equivalent
 * to the Iguana expression: v < 0
 */
Ig_API(int) IgNumber_IsNegative(IgObject *v);


/*
 * Sequence protocol interface
 * ===========================
 */

/*
 * Returns 1 if the object v provides sequence protocols, and
 * 0 otherwise.
 *
 * This macro always succeeds.
 */
#define IgSequence_TYPE(o)      ((o) && (o)->ob_type->tp_as_sequence)

/* 
 * Return the size of the sequence object s, or -1 on failure. This is the
 * equivalent of the Iguana expression: |s|;
 */
Ig_API(int) IgSequence_Size(IgObject *s);

/*
 * Assign object o to the i-th element of s. Returns -1 on failure. This is 
 * the equivalent of the Iguana statement: s[i]=o.
 */
Ig_API(int) IgSequence_SetItem(IgObject *s, IgObject *v, int i);

/*
 * Return the i-th element of s, or NULL on failure. This is the equivalent 
 * of the Iguana expression: s[i].
 */
Ig_API(IgObject *) IgSequence_GetItem(IgObject *s, int i);

/*
 * Determine if s contains the object o. If an item in s is equal to o, 
 * return 1, otherwise return 0. On error return -1. This is equivalent to the 
 * Iguana expression: o in s.
 */
Ig_API(int) IgSequence_Contains(IgObject *s, IgObject *o);

/*
 * Return the concatenation of sequences s1 and s2, or NULL on failure. 
 * This is the equivalent of the Iguana expression: s1 + s2.
 */
Ig_API(IgObject*) IgSequence_Concat(IgObject *s1, IgObject *s2);

/*
 * Return the concatenation of sequence s for n times, or NULL on failure.
 * This is the equivalent of the Iguana expression: v * n.
 */
Ig_API(IgObject*) IgSequence_Repeat(IgObject *s, int n);

#if defined(__cplusplus)
}				/* extern "C" { */
#endif

#endif				/* Ig_ABSTRACT_H */
