/* The Iguana top-level interface */
#ifndef Ig_IGUANARUN_H
#define Ig_IGUANARUN_H

#if defined(__cplusplus)
extern "C"
{
#endif

/*
 * Ig_Initialize() creates the fundamental modules '__builtin__', 
 * '__main__', 'sys', exceptions and modules table.
 *
 * The embedding application can set it's name by calling   
 * Ig_SetProgramName() *BEFORE* calling Ig_Initialize().
 */
Ig_API(void) Ig_Initialize(void);

/*  
 * Undo all initializations made by Ig_Initialize(). Ideally, this
 * frees all memory allocated by the Iguana interpreter.
 */
Ig_API(void) Ig_Finalize(void);

/*
 * Return 1 when the Iguana interpreter has been initialized, 
 * 0 if not. After Ig_Finalize() is  called, this returns false 
 * until Ig_Initialize() is called again. 
 */
Ig_API(int) Ig_IsInitialized(void);

/*
 * Terminate the iguana process. This calls Ig_Finalize() and then
 * calls the standard C library function exit(status).
 */
Ig_API(void) Ig_Exit(int) Ig_NORETURN;

/*
 * Register a cleanup function to be called by Ig_Finalize()  The
 * cleanup function will be called with no arguments and should
 * return no value. At most 32 cleanup functions can be
 * registered. When the registration is successful, Ig_AtExit()
 * returns 0; on failure, it returns -1. The cleanup function
 * registered last is called first.
 */
Ig_API(int) Ig_AtExit(void (*func) (void));

/*
 * Print a fatal error message and kill the interpreter. No cleanup is 
 * performed. This function should only be invoked when a condition is 
 * detected that would make it dangerous to continue.
 */
Ig_API(void) Ig_Suicide(const char *) Ig_NORETURN;

/* 
 * Return 1 if an interrupt was occurred before the last time that
 * this function was called. Otherwise return 0.
 */
Ig_API(int) Ig_Interrupted(void); 

/*
 * Ig_GETENV() is a wrapper around getenv() that pays attention to
 * Ig_IgnoreEnvironmentFlag. It should be used for getting
 * variables like IGUANAPROMPT and IGUANASILENT from the environment.
 */
#define Ig_GETENV(s) (Ig_IgnoreEnvironmentFlag ? NULL : getenv(s))

/* Environment flags and modes. */
Ig_API(int) Ig_IgnoreEnvironmentFlag;
Ig_API(int) Ig_InteractiveMode;
Ig_API(int) Ig_CommandMode;
Ig_API(int) Ig_VerboseMode;
Ig_API(int) Ig_WarningMode;

/* Utilities. */
Ig_API(const char *) Ig_GetPrompt(void);

Ig_API(void) Ig_SetProgramName(const char *);
Ig_API(const char *) Ig_GetProgramName(void);

/* In their own files. */
Ig_API(const char *) Ig_GetBuildInfo(void);
Ig_API(const char *) Ig_GetCopyright(void);
Ig_API(const char *) Ig_GetCompiler(void);
Ig_API(const char *) Ig_GetPath(void);
Ig_API(const char *) Ig_GetPlatform(void);
Ig_API(const char *) Ig_GetVersion(void);

/* Interfaces to parse and execute pieces of Iguana code. */
Ig_API(int) IgRun_String(const char *);
Ig_API(int) IgRun_File(const char *);
Ig_API(int) IgRun_Interactive(void);

#if defined(__cplusplus)
}				/* extern "C" { */
#endif

#endif				/* Ig_IGUANARUN_H */
