/* Object and Type Object interface */

/*
 * Objects are structures allocated on the heap, never statically or on the 
 * stack; they must be accessed through special macros and functions only.  
 * (Type objects are exceptions to the first rule; the standard types are 
 * represented by statically initialized structures. There is also few other
 * other object that are exceptions to the first rule such as Ig_True,
 * Ig_False and Ig_None).
 *
 * Objects are always accessed through pointers of the type 'IgObject *'.
 * The type 'IgObject' is a structure that contains only the reference count 
 * an the type pointer:
 *
 *    the object has a 'reference count' is increased or decreased when a 
 *    pointer to the object is copied or deleted: when it reaches zero there 
 *    are no references to the object left and it can be removed from the heap;
 *
 *    the object has a 'type' that determines what it represents and how it 
 *    implements a standard operation. An object's type is fixed when it is 
 *    created.
 *
 * Objects do not float around in memory; once allocated an object keeps
 * the same size and address. Objects that must hold variable-size data
 * can contain pointers to variable-size parts of the object. Not all
 * objects of the same type have the same size; but the size cannot change
 * after allocation.
 *
 * The actual memory allocated for an object contains other data that can 
 * only be accessed after casting the pointer to a pointer to a longer 
 * structure type. This longer type *MUST* start with the reference count and 
 * type fields: the implementation of a particular object type can cast the 
 * object pointer to the proper type and back.
 */

#ifndef Ig_OBJECT_H
#define Ig_OBJECT_H

#ifdef __cplusplus
extern "C"
{
#endif

typedef struct _IgNumberMethods IgNumberMethods;
typedef struct _IgSequenceMethods IgSequenceMethods;

typedef struct _IgTypeObject IgTypeObject;
typedef struct _IgObject IgObject;

/* IgObject_HEAD defines the initial segment of all the objects. */
#define IgObject_HEAD 		\
        int ob_refcnt; 		\
        IgTypeObject *ob_type

#define IgObject_VAR_HEAD	\
	IgObject_HEAD;		\
	int ob_size

/* 
 * Initialize an Iguana object. Set the reference counter to 1 and set the 
 * object's type to the pointer to type object. This should be used only on 
 * freshly allocated objects.
 */
#define IgObject_INIT(type)	\
	1, &type

/* 
 * The object structure: any Iguana object can be cast to a IgObject*. This 
 * is inheritance built by hand.  
 */
struct _IgObject
{
	IgObject_HEAD;
};

/* Prototypes for the numeric and sequential protocols. */
typedef IgObject *(*unaryfunc) (IgObject *);
typedef IgObject *(*binaryfunc) (IgObject *, IgObject *);
typedef IgObject *(*ternaryfunc) (IgObject *, IgObject *, IgObject *);
typedef int (*inquiryfunc) (IgObject *);
typedef int (*coercefunc) (IgObject **, IgObject **);
typedef int (*setitemfunc) (IgObject *, IgObject *, int);
typedef IgObject *(*getitemfunc) (IgObject *, int);
typedef int (*containsfunc) (IgObject *, IgObject *);
typedef IgObject *(*concatfunc) (IgObject *, IgObject *);
typedef IgObject *(*repeatfunc) (IgObject *, int);

/* Numeric methods. */
struct _IgNumberMethods
{
	coercefunc nm_coerce;
	binaryfunc nm_add;
	binaryfunc nm_sub;
	binaryfunc nm_mul;
	binaryfunc nm_true_div;
	binaryfunc nm_floor_div;
	binaryfunc nm_rem;
	binaryfunc nm_pow;
	unaryfunc nm_neg;
	unaryfunc nm_pos;
	unaryfunc nm_abs;
	inquiryfunc nm_iszero;
	inquiryfunc nm_ispos;
	inquiryfunc nm_isneg;
};

/* Sequential methods. */
struct _IgSequenceMethods
{
	inquiryfunc sq_size;
	setitemfunc sq_setitem;
	getitemfunc sq_getitem;
	containsfunc sq_contains;
	concatfunc sq_concat;
	repeatfunc sq_repeat;	
};

/* Prototypes for the type object methods. */
typedef IgObject *(*newfunc) (IgObject *);
typedef int (*delfunc) (IgObject *);
typedef int (*printfunc) (IgObject *, FILE *);
typedef IgObject *(*reprfunc) (IgObject *);

/* For objects comparison. */
typedef enum
{
	IgCmp_LT,	/*  < */
	IgCmp_GT,	/*  > */
	IgCmp_LE,	/* >= */
	IgCmp_GE,	/* <= */
	IgCmp_NE,	/* != */
	IgCmp_EQ	/* == */
} IgCmp;

typedef int (*cmpfunc) (IgObject *, IgObject *, IgCmp);

/*
 * Type objects contain a string containing the type name, the documentation
 * string, the size of the object structure (for allocation), and methods as
 * function pointers. Methods are optional, a NULL pointer meaning that 
 * particular kind of access is not available for this type.
 */

struct _IgTypeObject
{
	IgObject_HEAD;
	char *tp_name;
	char *tp_doc;
	unsigned int tp_size;

	/* Methods to implement standard operations. */
	newfunc	tp_new;
	delfunc tp_del;
	printfunc tp_print;
	reprfunc tp_repr;
	cmpfunc tp_cmp;

	/* Protocols. */
	IgNumberMethods *tp_as_number;
	IgSequenceMethods *tp_as_sequence;
};

/* This instance of IgTypeObject represents the Iguana type object itself. */
#define IgType_TYPE(obj)        ((obj)->ob_type == &IgType_Type)

/* Cast the object pointer passed as argument to a IgTypeObject pointer. */
#define IgType_CAST(obj)	((IgTypeObject *) obj)

/* This instance of IgTypeObject represents the type object itself. */
Ig_API(IgTypeObject) IgType_Type;


/*
 * The macros Ig_INCREF(obj) and Ig_DECREF(obj) are used to increment or 
 * decrement reference counter. Ig_DECREF calls the object's del method when
 * the ob_refcnt is 0: for objects that don't contain references to other 
 * objects or heap memory this can be the standard function free().  
 * Both macros can be used wherever a void expression is allowed. The argument 
 * shouldn't be a NULL pointer, however, there are also two other 
 * implementation * of Ig_INCREF and Ig_DECREF that can accept a NULL pointer.
 * This macros are Ig_XINCREF and Ig_XDECREF.
 *
 * We assume that the reference count field can never overflow; this can
 * be proven when the size of the field is the same as the pointer size
 * but even with a 16-bit reference count field it is pretty unlikely so
 * we ignore the possibility, if you are paranoid, make it a long... :-)
 *
 * Type objects should never be deallocated; the type pointer in an object
 * is not considered to be a reference to the type object, to save
 * complications in the deallocation function.  (This is actually a
 * decision that's up to the implementer of each new type so if you want,
 * you can count such references to the type object.)
 */

#define Ig_INCREF(obj) \
	((obj)->ob_refcnt++)

#define Ig_DECREF(obj) \
        if (--(obj)->ob_refcnt == 0) \
                IgObject_Del((IgObject *)(obj))

/* Macros to use in case the object pointer may be NULL. */
#define Ig_XINCREF(obj) \
        if ((obj) != NULL) \
                Ig_INCREF((IgObject *)(obj))

#define Ig_XDECREF(obj) \
        if ((obj) != NULL) \
                Ig_DECREF((IgObject *)(obj))

#ifdef __cplusplus
}
#endif

#endif				/* Ig_OBJECT_H */
