/* 
 * Symbols and macros to supply platform-independent interfaces to basic C
 * language and library operations whose differ across platforms.
 */
#ifndef Ig_SYSTEM_H
#define Ig_SYSTEM_H

#ifndef HAVE_STDARG_PROTOTYPES
# error "Iguana needs ANSI C89 stdarg macros"   /* XXX: the right remedy? */
#endif

#ifndef HAVE_PROTOTYPES
# error "Iguana needs ANSI C prototypes"        /* XXX: ditto */
#endif

#include <stdio.h>

#if HAVE_STDARG_H
# include <stdarg.h>
#endif

#if HAVE_SYS_TYPES_H
# include <sys/types.h>
#endif

#if STDC_HEADERS
# include <stdlib.h>
# include <stddef.h>
#else
# if HAVE_STDLIB_H
#  include <stdlib.h>
# endif
#endif

#if HAVE_STRING_H
# if !STDC_HEADERS && HAVE_MEMORY_H
#  include <memory.h>
# endif
# include <string.h>
#endif

#if HAVE_STRINGS_H
# include <strings.h>
#endif

#include <errno.h>
#include <math.h>

#if HAVE_FLOAT_H
# include <float.h>             /* for DBL_EPSILON, DBL_MIN, DBL_MAX */
#endif

/* portable ctype.h macros */
#include <ctype.h>

#define IgSys_ISASCII(c)        isascii((int)(c))

#define IgSys_ISALPHA(c)	(IgSys_ISASCII(c) && isalpha((int)(c)))
#define IgSys_ISDIGIT(c)	(IgSys_ISASCII(c) && isdigit((int)(c)))
#define IgSys_ISALNUM(c)	(IgSys_ISASCII(c) && isalnum((int)(c)))
#define IgSys_ISSPACE(c)	(IgSys_ISASCII(c) && isspace((int)(c)))
#define IgSys_ISUPPER(c)	(IgSys_ISASCII(c) && isupper((int)(c)))
#define IgSys_ISLOWER(c)	(IgSys_ISASCII(c) && islower((int)(c)))

/*
 * Provide definitions for some commonly used macros. Some of them are only 
 * provided if they haven't already been defined: it's assumed that if they 
 * are already defined then the current definition is correct.
 */
#ifndef NULL
# ifdef __cplusplus
#  define NULL        (0L)
# else
#  define NULL        ((void*) 0)
# endif
#endif

/* 
 * The following test is to work around the gross typo in systems 
 * whereby EXIT_FAILURE is defined to 0, not 1.  
 */
#if !EXIT_FAILURE
# undef EXIT_FAILURE
# define EXIT_FAILURE 1
#endif

#ifndef EXIT_SUCCESS
# define EXIT_SUCCESS 0
#endif

/*
 * Want to use __attribute__ keyword (gcc extension) without upsetting other
 * compilers: put any required defines magic into configure.ac!
 */
#ifdef NORETURN 
# define Ig_NORETURN    NORETURN
#else
# define Ig_NORETURN
#endif

#if !defined(__GNUC__) || __GNUC__ < 2 || (__GNUC__ == 2 && __GNUC_MINOR__ < 7)
# define Ig_ATTRIBUTE(x)
#else
# define Ig_ATTRIBUTE(x) __attribute__(x)
#endif

/* limits.h constants that may be missing */
#ifndef INT_MAX
# define INT_MAX       2147483647
#endif

#ifndef INT_MIN
# define INT_MIN       (-INT_MAX-1)
#endif

#ifndef LONG_MAX
# if SIZEOF_LONG == 4
#  define LONG_MAX 	0X7FFFFFFFL
# elif SIZEOF_LONG == 8
#  define LONG_MAX 	0X7FFFFFFFFFFFFFFFL
# else
#  error "could not set LONG_MAX"
# endif
#endif

#ifndef LONG_MIN
# define LONG_MIN 	(-LONG_MAX-1)
#endif

/* ANSI C defines for string manipulation functions */
#ifndef HAVE_STRCHR
# define strchr 	index
# define strrchr 	rindex
#endif

/* Declare functions replacements (if needed) */
#ifndef HAVE_ISATTY
Ig_API(int) isatty(int); /* Iguana/isatty.c */
#endif

/* 
 * NOTE: neither bcopy() nor the memcpy() implementations can reliably 
 *       handle copying in overlapping areas of memory. Use memmove 
 *       if you need that behaviour.  
 */
#ifndef HAVE_MEMCPY
# ifdef HAVE_BCOPY
#  define memcpy(d, s, n)  	bcopy ((s), (d), (n))
# else
Ig_API(void *) memcpy(void *, const void *, size_t); /* Iguana/memcpy.c */
# endif
#endif

#ifndef HAVE_MEMMOVE
# ifdef HAVE_BCOPY
#  define memmove(d, s, n) 	bcopy ((s), (d), (n))
# else
Ig_API(void *) memmove(void *, const void *, size_t); /* Iguana/memmove.c */
# endif
#endif

#ifndef HAVE_STRCHR
# ifdef HAVE_INDEX
#  define strchr 		index
# else
Ig_API(const char *) strchr(const char *, int ); /* Iguana/strchr.c */
# endif
#endif

#ifndef HAVE_STRRCHR
# if HAVE_RINDEX
#  define strrchr 		rindex
# else
Ig_API(const char *) strrchr(const char *, int ); /* Iguana/strrchr.c */
# endif
#endif 

#ifndef HAVE_STRCMP
Ig_API(int) strcmp(const char *, const char *);	/* Iguana/strcmp.c */
#endif

#ifndef HAVE_STRERROR
Ig_API(char *) strerror(int); /* Iguana/strerror.c */
#endif

#ifndef HAVE_STRTOD
Ig_API(double) strtod(const char *, char **); /* Iguana/strtod.c */
#endif

#ifndef HAVE_STRTOUL
Ig_API(unsigned long) strtoul(const char *, char *, int); /* Iguana/strtol.c */
#endif

#ifndef HAVE_STRTOL
Ig_API(long) strtol(const char *, char **, int); /* Iguana/strtol.c */
#endif

#if defined(__cplusplus)
extern "C"
{
#endif

/* Getopt interface */
Ig_API(int) IgSys_opterr;
Ig_API(int) IgSys_optind;
Ig_API(char *) IgSys_optarg;

Ig_API(int) IgSys_getopt(int, char **, char *);

/*
 * The native C implementation for snprintf/vsnprintf function are
 * not available on all platforms, which is why we provide emulations
 * for those platforms in Iguana/mysnprintf.c
 *
 * BEWARE: the return value of snprintf varies across platforms; do
 * not rely on any particular behavior; eventually the C99 defs may
 * be reliable.
 */
Ig_API(int) IgSys_vsnprintf(char *, size_t, const char *, va_list);
Ig_API(int) IgSys_snprintf(char *, size_t, const char *, ...)
	                       Ig_ATTRIBUTE((format(printf, 3, 4)));

/* Wrappers around sigaction() or signal(). */
typedef RETSIGTYPE (*IgSys_sighandler_t) (int);

Ig_API(IgSys_sighandler_t) IgSys_getsignal(int);
Ig_API(IgSys_sighandler_t) IgSys_setsignal(int, IgSys_sighandler_t);

/* Hook for readline function. */
typedef char *(*IgSys_readline_t) (const char *);

Ig_API(void) IgSys_setreadline(IgSys_readline_t);
Ig_API(IgSys_readline_t) IgSys_getreadline(void);

/* Wrappers around random(), lrand48() and rand(). */
Ig_API(long) IgSys_random(void);
Ig_API(long) IgSys_random_range(long, long);
Ig_API(long) IgSys_srandom(long);

/* 
 * Raw memory interface 
 * ====================
 */

/*
 * Functions
 *
 * Functions supplying platform-independent semantics for
 * malloc/realloc/free. These functions make sure that allocating
 * 0 bytes returns a distinct non-NULL pointer (whenever possible, if
 * we are flat out of memory, NULL may be returned), even if the
 * platform malloc and realloc don't.
 *
 * IgSys_malloc(0) means malloc(1). Some systems would return NULL
 * for malloc(0), which would be treated as an error. Some platforms
 * would return a pointer with no memory behind it, which would break
 * malloc. To solve these problems, allocate an extra byte. 
 */
Ig_API(void *) IgSys_malloc(int);
Ig_API(void *) IgSys_realloc(void *, int);

/* free() replacement. It check for NULL before call free(). */
Ig_API(void) IgSys_free(void *);

/*
 * strdup() replacement. It never returns NULL, even if you pass it 
 * as parameter ("<NULL>" is returned, instead). 
 */ 
Ig_API(char *) IgSys_strdup(const char *); /* Added in Iguana 0.1.3 */

/*
 * Macros
 *
 * Provide macros for easily allocating memory. The macros will cast the
 * allocated memory to the specified type in order to avoid compiler warnings.
 */

#define IgMem_NEW(type, count) \
        (type *) IgSys_malloc ((unsigned) sizeof (type) * (count))

#define IgMem_REALLOC(type, mem, count) \
        (type *) IgSys_realloc (mem, (unsigned) sizeof (type) * (count))

#define IgMem_FREE(mem) \
	IgSys_free((mem))

#if defined(__cplusplus)
}				/* extern "C" { */
#endif

#endif				/* Ig_SYSTEM_H */
